with Ada.Text_IO; use Ada.Text_IO;
with Mast_Parser, Mast_Results_Parser;

package body Mast_Actions is

   type String_Ptr is access String;

   System_Name, Results_Name : String_Ptr;
   System_Present, Results_Present : Boolean:=False;

   -----------------
   -- Has_Results --
   -----------------

   function Has_Results return Boolean is
   begin
      return Results_Present;
   end Has_Results;

   ----------------
   -- Has_System --
   ----------------

   function Has_System return Boolean is
   begin
      return System_Present;
   end Has_System;

   ---------------------
   -- Name_Of_Results --
   ---------------------

   function Name_Of_Results return String is
   begin
      if Results_Present then
         return Results_Name.all;
      else
         raise No_Results;
      end if;
   end Name_Of_Results;

   --------------------
   -- Name_Of_System --
   --------------------

   function Name_Of_System return String is
   begin
      if System_Present then
         return System_Name.all;
      else
         raise No_System;
      end if;
   end Name_Of_System;

   ------------------
   -- Read_Results --
   ------------------

   procedure Read_Results (Filename : String) is
      F : Ada.Text_IO.File_Type;
   begin
      if System_Name=null then
         raise No_System;
      end if;
      Results_Name:=new String'(Filename);
      Ada.Text_IO.Open(F,Ada.Text_IO.In_File,Filename);
      begin
         Ada.Text_IO.Set_Input(F);
         Mast_Results_Parser(The_System);
         Close(F);
         Ada.Text_IO.Set_Input(Standard_Input);
         Results_Present:=True;
      exception
         when others =>
            Close(F);
            Ada.Text_IO.Set_Input(Standard_Input);
            raise;
      end;
   end Read_Results;

   -----------------
   -- Read_System --
   -----------------

   procedure Read_System (Filename : String) is
      F : Ada.Text_IO.File_Type;
   begin
      System_Name:=new String'(Filename);
      Ada.Text_IO.Open(F,Ada.Text_IO.In_File,Filename);
      begin
         Ada.Text_IO.Set_Input(F);
         Mast_Parser(The_System);
         Close(F);
         Ada.Text_IO.Set_Input(Standard_Input);
         System_Present:=True;
      exception
         when others =>
            Close(F);
            Ada.Text_IO.Set_Input(Standard_Input);
            raise;
      end;
   end Read_System;

   ------------------
   -- Save_Results --
   ------------------

   procedure Save_Results (Filename : String) is
      Out_Res : File_Type;
   begin
      Ada.Text_IO.Create(Out_Res,Ada.Text_IO.Out_File,Filename);
      MAST.Systems.Print_Results(Out_Res,The_System);
      Close(Out_Res);
   end Save_Results;

   -----------------
   -- Save_System --
   -----------------

   procedure Save_System (Filename : String) is
      Out_Res : File_Type;
   begin
      Ada.Text_IO.Create(Out_Res,Ada.Text_IO.Out_File,Filename);
      MAST.Systems.Print(Out_Res,The_System);
      Close(Out_Res);
   end Save_System;

end Mast_Actions;
