-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000-2002                     --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@unican.es                     --
--          Jose Javier Gutierrez  gutierjj@unican.es                --
--          Jose Carlos Palencia   palencij@unican.es                --
--          Jose Maria Drake       drakej@unican.es                  --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with MAST.Processing_Resources, MAST.Scheduling_Parameters,
  MAST.Results, Named_Lists, Var_Strings,Ada.Text_IO;

package MAST.Scheduling_Servers is

   type Scheduling_Server is abstract tagged private;

   procedure Init (Srvr : in out Scheduling_Server;
                   Name : Var_Strings.Var_String);
   function Name (Srvr : Scheduling_Server)
                 return Var_Strings.Var_String;

   procedure Set_Server_Processing_Resource
     (Srvr : in out Scheduling_Server;
      The_Server_Processing_Resource :
      MAST.Processing_Resources.Processing_Resource_Ref);
   function Server_Processing_Resource
     (Srvr : Scheduling_Server)
     return MAST.Processing_Resources.Processing_Resource_Ref;

   procedure Set_Server_Sched_Parameters
     (Srvr : in out Scheduling_Server;
      The_Server_Sched_Parameters :
      MAST.Scheduling_Parameters.Sched_Parameters_Ref);
   function Server_Sched_Parameters
     (Srvr : Scheduling_Server)
     return MAST.Scheduling_Parameters.Sched_Parameters_Ref;

   procedure Set_Sched_Params_Result
     (Sch : in out Scheduling_Server;
      Res : Results.Sched_Params_Result_Ref);

   function Sched_Params_Result
     (Sch : Scheduling_Server)
     return Results.Sched_Params_Result_Ref;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Scheduling_Server;
      Indentation : Positive;
      Finalize    : Boolean:=False;
      Complete    : Boolean:=True) is abstract;

   procedure Print_Results
     (File : Ada.Text_IO.File_Type;
      Sch  : Scheduling_Server;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   type Scheduling_Server_Ref is access Scheduling_Server'Class;

   function Clone
     (Srvr : Scheduling_Server)
     return  Scheduling_Server_Ref is abstract;

   function Name (Srvr_Ref : Scheduling_Server_Ref )
                 return Var_Strings.Var_String;

   package Lists is new Named_Lists
     (Element => Scheduling_Server_Ref,
      Name    => Name);

   function Clone
     (The_List : in Lists.List)
     return Lists.List;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_List : in out Lists.List;
      Indentation : Positive);

   procedure Print_Results
     (File : Ada.Text_IO.File_Type;
      The_List : in out Lists.List;
      Indentation : Positive);

   type Fixed_Priority_Server is new Scheduling_Server with private;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Res : in out Fixed_Priority_Server;
      Indentation : Positive;
      Finalize    : Boolean:=False;
      Complete    : Boolean:=True);

   function Clone
     (Srvr : Fixed_Priority_Server)
     return  Scheduling_Server_Ref;
   -- Clone is incomplete. The processing resource reference points at
   -- the old system
   -- This field needs later adjustment

   function Get_Min_System_Priority
     (The_List : Lists.List)
     return Priority;
   -- raises Incorrect object if servers are not Fixed_Priority_Server


private

   type Scheduling_Server is abstract tagged record
      Name : Var_Strings.Var_String;
      Server_Processing_Resource :
        MAST.Processing_Resources.Processing_Resource_Ref;
      Server_Sched_Parameters :
        MAST.Scheduling_Parameters.Sched_Parameters_Ref;
      The_Sched_Params_Result : Results.Sched_Params_Result_Ref;
   end record;

   type Fixed_Priority_Server is new Scheduling_Server with null record;

end MAST.Scheduling_Servers;
