-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000-2002                     --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@unican.es                     --
--          Jose Javier Gutierrez  gutierjj@unican.es                --
--          Jose Carlos Palencia   palencij@unican.es                --
--          Jose Maria Drake       drakej@unican.es                  --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------
with Ada.Text_IO;
package body Mast.Systems is

   use type Results.Slack_Result_Ref;
   use type Results.Trace_Result_Ref;

   -----------
   -- Clone --
   -----------

   function Clone(The_System : System) return System
   is
      The_Copy : System;
   begin
      declare
         Incomplete : exception;
      begin
         raise Incomplete;
      end;
      -- Clone the basic objects.
      -- Internal pointers will continue pointing at the old system
      The_Copy:=
        (Model_Name           => The_System.Model_Name,
         Model_Date           => The_System.Model_Date,
         Generation_Tool      => The_System.Generation_Tool,
         Generation_Profile   => The_System.Generation_Profile,
         Generation_Date      => The_System.Generation_Date,
         Processing_Resources =>
           Processing_Resources.Clone(The_System.Processing_Resources),
         Shared_Resources     =>
           Shared_Resources.Clone(The_System.Shared_Resources),
         Operations           =>
           Operations.Clone(The_System.Operations),
         Transactions         =>
           Transactions.Clone(The_System.Transactions),
         Scheduling_Servers   =>
           Scheduling_Servers.Clone(The_System.Scheduling_Servers),
         The_Slack_Result     => The_System.The_Slack_Result,
         The_Trace_Result     => The_System.The_Trace_Result);

      -- rearrange all the internal pointers in simple operations
      Operations.Adjust(The_Copy.Operations,The_Copy.Shared_Resources);

      -- rearrange all the internal pointers in scheduling servers
      --Scheduling_Servers.Adjust(The_Copy.Scheduling_Servers,
      --                          The_Copy.Processing_Resources);

      -- rearrange all the internal pointers in network drivers:

      -- rearrange all the internal pointers in transactions:
      --       timing requirements, links, event handlers

      return The_Copy;
   end Clone;


   ----------------------
   -- Clone_Operations --
   ----------------------

   function Clone_Operations(The_System : System) return System
   is
      The_Copy : System;
   begin
      -- Clone the basic objects.
      -- Internal pointers will continue pointing at the old system
      The_Copy:=
        (Model_Name           => The_System.Model_Name,
         Model_Date           => The_System.Model_Date,
         Generation_Tool      => The_System.Generation_Tool,
         Generation_Profile   => The_System.Generation_Profile,
         Generation_Date      => The_System.Generation_Date,
         Processing_Resources => The_System.Processing_Resources,
         Shared_Resources     => The_System.Shared_Resources,
         Operations           => Operations.Clone(The_System.Operations),
         Transactions         => The_System.Transactions,
         Scheduling_Servers   => The_System.Scheduling_Servers,
         The_Slack_Result     => The_System.The_Slack_Result,
         The_Trace_Result     => The_System.The_Trace_Result);

      -- rearrange the internal pointers referencing operations
      Transactions.Adjust_Operations
        (The_Copy.Transactions,The_Copy.Operations);


      return The_Copy;
   end Clone_Operations;

   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      The_System : in out System;
      Indentation : Positive:=1)
   is
      Name_Length : constant := 10;
   begin
      if The_System.Model_Name/=Null_Var_String or
        The_System.Model_Date/="                   "
      then
         Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation));
         Ada.Text_IO.Put(File,"Model (");
         if The_System.Model_Name/=Null_Var_String then
            MAST.IO.Print_Arg
              (File,"Model_Name",
               IO.Name_Image(The_System.Model_Name),Indentation+3,Name_Length);
         end if;
         if The_System.Model_Date/="                   " then
            MAST.IO.Print_Separator(File);
            MAST.IO.Print_Arg
              (File,"Model_Date",MAST.IO.Date_Image(The_System.Model_Date),
               Indentation+3,Name_Length);
         end if;
         MAST.IO.Print_Separator(File,",",True);
         Ada.Text_IO.New_Line(File);
      end if;
      MAST.Processing_Resources.Print
        (File,The_System.Processing_Resources,Indentation);
      MAST.Shared_Resources.Print
        (File,The_System.Shared_Resources,Indentation);
      MAST.Operations.Print
        (File,The_System.Operations,Indentation);
      MAST.Scheduling_Servers.Print
        (File,The_System.Scheduling_Servers,Indentation);
      MAST.Transactions.Print
        (File,The_System.Transactions,Indentation);
   end Print;

   -------------------
   -- Print_Results --
   -------------------

   procedure Print_Results
     (File : Ada.Text_IO.File_Type;
      The_System : in out System;
      Indentation : Positive:=1)
   is
      Names_Length : constant := 18;
   begin

      -- Print system-specific results

      Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation));
      Ada.Text_IO.Put(File,"Real_Time_Situation (");
      if The_System.Model_Name/=Null_Var_String then
         MAST.IO.Print_Arg
           (File,"Model_Name",
            IO.Name_Image(The_System.Model_Name),Indentation+3,Names_Length);
         MAST.IO.Print_Separator(File);
      end if;
      if The_System.Model_Date/="                   " then
         MAST.IO.Print_Arg
           (File,"Model_Date",
            IO.Date_Image(The_System.Model_Date),Indentation+3,Names_Length);
         MAST.IO.Print_Separator(File);
      end if;
      MAST.IO.Print_Arg
        (File,"Generation_Tool",
         """"&To_String(The_System.Generation_Tool)&"""",
         Indentation+3,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Generation_Profile",
         """"&To_String(The_System.Generation_Profile)&"""",
         Indentation+3,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Generation_Date",
         MAST.IO.Date_Image(The_System.Generation_Date),
         Indentation+3,Names_Length);
      if The_System.The_Trace_Result/=null or else
        The_System.The_Slack_Result/=null
      then
         MAST.IO.Print_Separator(File);
         MAST.IO.Print_Arg
           (File,"Results","",Indentation+3,Names_Length);
         MAST.IO.Print_Separator(File,MAST.IO.Nothing);
         MAST.IO.Print_List_Item(File,MAST.IO.Left_Paren,
                                 Indentation+7);
         if The_System.The_Slack_Result/=null then
            Results.Print(File,The_System.The_Slack_Result.all,Indentation+8);
            if The_System.The_Trace_Result/=null then
               MAST.IO.Print_Separator(File);
            end if;
         end if;
         if The_System.The_Trace_Result/=null then
            Results.Print(File,The_System.The_Trace_Result.all,Indentation+8);
         end if;
         Ada.Text_IO.Put(File,")");
      end if;
      MAST.IO.Print_Separator(File,MAST.IO.Nothing,True);
      Ada.Text_IO.New_Line(File);

      -- Now print rest of results

      MAST.Transactions.Print_Results
        (File,The_System.Transactions,Indentation);

      MAST.Processing_Resources.Print_Results
        (File,The_System.Processing_Resources,Indentation);

      MAST.Operations.Print_Results
        (File,The_System.Operations,Indentation);

      MAST.Shared_Resources.Print_Results
        (File,The_System.Shared_Resources,Indentation);

      MAST.Scheduling_Servers.Print_Results
        (File,The_System.Scheduling_Servers,Indentation);

   end Print_Results;

   ----------------------
   -- Set_Slack_Result --
   ----------------------

   procedure Set_Slack_Result
     (Sys : in out System; Res : Results.Slack_Result_Ref)
   is
   begin
      Sys.The_Slack_Result:=Res;
   end Set_Slack_Result;

   ----------------------
   -- Slack_Result     --
   ----------------------

   function Slack_Result
     (Sys : System) return Results.Slack_Result_Ref
   is
   begin
      return Sys.The_Slack_Result;
   end Slack_Result;

   ----------------------
   -- Set_Trace_Result --
   ----------------------

   procedure Set_Trace_Result
     (Sys : in out System; Res : Results.Trace_Result_Ref)
   is
   begin
      Sys.The_Trace_Result:=Res;
   end Set_Trace_Result;

   ----------------------
   -- Trace_Result     --
   ----------------------

   function Trace_Result
     (Sys : System) return Results.Trace_Result_Ref
   is
   begin
      return Sys.The_Trace_Result;
   end Trace_Result;


end Mast.Systems;
