with Mast.IO;

package body Mast.Timers is

   ------------------
   -- Avg_Overhead --
   ------------------

   function Avg_Overhead
     (Timer : System_Timer)
     return Time
   is
   begin
      return Timer.Avg_Overhead;
   end Avg_Overhead;

   -------------------
   -- Best_Overhead --
   -------------------

   function Best_Overhead
     (Timer : System_Timer)
     return Time
   is
   begin
      return Timer.Best_Overhead;
   end Best_Overhead;

   ------------
   -- Clone --
   ------------

   function Clone
     (Timer : Alarm_Clock)
     return System_Timer_Ref
   is
      Timer_Ref : System_Timer_Ref;
   begin
      Timer_Ref:=new Alarm_Clock'(Timer);
      return Timer_Ref;
   end Clone;

   ------------
   -- Clone --
   ------------

   function Clone
     (Timer : Ticker)
     return System_Timer_Ref
   is
      Timer_Ref : System_Timer_Ref;
   begin
      Timer_Ref:=new Ticker'(Timer);
      return Timer_Ref;
   end Clone;


   ------------
   -- Period --
   ------------

   function Period
     (Timer : Ticker)
     return Time
   is
   begin
      return Timer.Period;
   end Period;

   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Timer : in out System_Timer;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
   begin
      Ada.Text_IO.Set_Col(File,Ada.Text_IO.Count(Indentation));
      Ada.Text_IO.Put(File,"(");
   end Print;

   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Timer : in out Alarm_Clock;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant := 14;
   begin
      Print(File,System_Timer(Timer),Indentation);
      MAST.IO.Print_Arg
        (File,"Type",
         "Alarm_Clock",Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Worst_Overhead",
         IO.Time_Image(Timer.Worst_Overhead),Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Avg_Overhead",
         IO.Time_Image(Timer.Avg_Overhead),Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Best_Overhead",
         IO.Time_Image(Timer.Best_Overhead),Indentation+2,Names_Length);
      Ada.Text_IO.Put(File,")");
   end Print;

   -----------
   -- Print --
   -----------

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Timer : in out Ticker;
      Indentation : Positive;
      Finalize    : Boolean:=False)
   is
      Names_Length : constant := 14;
   begin
      Print(File,System_Timer(Timer),Indentation);
      MAST.IO.Print_Arg
        (File,"Type",
         "Ticker",Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Worst_Overhead",
         IO.Time_Image(Timer.Worst_Overhead),Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Avg_Overhead",
         IO.Time_Image(Timer.Avg_Overhead),Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Best_Overhead",
         IO.Time_Image(Timer.Best_Overhead),Indentation+2,Names_Length);
      MAST.IO.Print_Separator(File);
      MAST.IO.Print_Arg
        (File,"Period",
         IO.Time_Image(Timer.Period),Indentation+2,Names_Length);
      Ada.Text_IO.Put(File,")");
   end Print;

   ----------------------
   -- Set_Avg_Overhead --
   ----------------------

   procedure Set_Avg_Overhead
     (Timer : in out System_Timer;
      Overhead : in Time)
   is
   begin
      Timer.Avg_Overhead:=Overhead;
   end Set_Avg_Overhead;

   -----------------------
   -- Set_Best_Overhead --
   -----------------------

   procedure Set_Best_Overhead
     (Timer : in out System_Timer;
      Overhead : in Time)
   is
   begin
      Timer.Best_Overhead:=Overhead;
   end Set_Best_Overhead;

   ----------------
   -- Set_Period --
   ----------------

   procedure Set_Period
     (Timer : in out Ticker;
      The_Period : in Time)
   is
   begin
      Timer.Period:=The_Period;
   end Set_Period;

   ------------------------
   -- Set_Worst_Overhead --
   ------------------------

   procedure Set_Worst_Overhead
     (Timer : in out System_Timer;
      Overhead : in Time)
   is
   begin
      Timer.Worst_Overhead:=Overhead;
   end Set_Worst_Overhead;

   --------------------
   -- Worst_Overhead --
   --------------------

   function Worst_Overhead
     (Timer : System_Timer)
     return Time
   is
   begin
      return Timer.Worst_Overhead;
   end Worst_Overhead;

end Mast.Timers;

