with Ada.Text_IO;
package Mast.Timers is

   type System_Timer is abstract tagged private;

   procedure Set_Worst_Overhead
     (Timer : in out System_Timer;
      Overhead : in Time);
   procedure Set_Avg_Overhead
     (Timer : in out System_Timer;
      Overhead : in Time);
   procedure Set_Best_Overhead
     (Timer : in out System_Timer;
      Overhead : in Time);
   function Worst_Overhead
     (Timer : System_Timer) return Time;
   function Avg_Overhead
     (Timer : System_Timer) return Time;
   function Best_Overhead
     (Timer : System_Timer) return Time;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Timer : in out System_Timer;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   type System_Timer_Ref is access System_Timer'Class;

   function Clone
     (Timer : System_Timer)
     return System_Timer_Ref is abstract;

   ---------------------
   -- Alarm_Clock
   ---------------------

   type Alarm_Clock is new System_Timer with private;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Timer : in out Alarm_Clock;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   function Clone
     (Timer : Alarm_Clock)
     return System_Timer_Ref;

   ---------------------
   -- Ticker
   ---------------------

   type Ticker is new System_Timer with private;

   procedure Set_Period
     (Timer : in out Ticker;
      The_Period : in Time);

   function Period
     (Timer : Ticker) return Time;

   procedure Print
     (File : Ada.Text_IO.File_Type;
      Timer : in out Ticker;
      Indentation : Positive;
      Finalize    : Boolean:=False);

   function Clone
     (Timer : Ticker)
     return System_Timer_Ref;

private

   type System_Timer is abstract tagged record
      Worst_Overhead,
      Avg_Overhead,
      Best_Overhead : Time:=0.0;
   end record;

   type Alarm_Clock is new System_Timer with null record;

   type Ticker is new System_Timer with record
      Period : Time:=Large_Time;
   end record;

end Mast.Timers;




